#-------------------------------- grafxlib0.py ---------------------------------
# Graphics library based on Tkinter
#-------------------------------------------------------------------------------
from math import *
from tkinter import *

root = Tk()                                              # create Tk root widget
root.title("grafxlib0")

def Nint(x): return int(floor(x + 0.5))                        # nearest integer

#===============================================================================
def MainLoop():                                          # creates Tk event loop
   root.mainloop()

#===============================================================================
def GraphInit(nx = 800, ny = 600):                       # creates canvas widget
   global win, nxwin, nywin                             # canvas object and size

   nxwin = nx; nywin = ny                                          # canvas size
   win = Canvas(root, width = nxwin, height = nywin, bg = "white")
   win.pack()                                              # make canvas visible
   return win                                             # return canvas object

#===============================================================================
def Plot0(x, y, n, col = "blue",
          fxmin = 0.15, fxmax = 0.95, fymin = 0.15, fymax = 0.90,
          xtext = "x", ytext = "y", title = None):
#-------------------------------------------------------------------------------
#  Plots a real function of one variable specified by a set of (x, y) points.
#
#  x[]   - abscissas of tabulation points (x[1] through x[n])
#  y[]   - ordinates of tabulation points (y[1] through y[n])
#  n     - number of tabulation points
#  col   - plot color ("red", "green", "blue" etc.)
#  fxmin - min fractional x-limit of viewport (0 < fxmin < fxmax < 1)
#  fxmax - max fractional x-limit of viewport
#  fymin - min fractional y-limit of viewport (0 < fymin < fymax < 1)
#  fymax - max fractional y-limit of viewport
#  xtext - x-axis title
#  ytext - y-axis title
#  title - plot title
#-------------------------------------------------------------------------------
   global win, nxwin, nywin                             # canvas object and size

   xmin = min(x[1: n+1]); xmax = max(x[1: n+1])             # user domain limits
   ymin = min(y[1: n+1]); ymax = max(y[1: n+1])
                                              # corrections for horizontal plots
   if (ymin == 0.0 and ymax == 0.0): ymin = -1e0; ymax = 1e0
   if (ymin == ymax): ymin *= 0.9; ymax *= 1.1
                                                               # viewport limits
   ixmin = Nint(fxmin * nxwin); iymin = Nint((1.0-fymin) * nywin)
   ixmax = Nint(fxmax * nxwin); iymax = Nint((1.0-fymax) * nywin)
   win.create_rectangle(ixmin, iymax, ixmax, iymin)                 # draw frame
                                                    # axis labels and plot title
   win.create_text((ixmin+ixmax)/2, iymin+10, text = xtext, anchor = N)
   win.create_text(ixmin-10, (iymin+iymax)/2, text = ytext, anchor = E)
   win.create_text((ixmin + ixmax)/2, iymax - 10, text = title, anchor = S)
                                                               # label axes ends
   win.create_text(ixmin, iymin+10, text = "%5.2f" % xmin, anchor = NW)
   win.create_text(ixmax, iymin+10, text = "%5.2f" % xmax, anchor = NE)
   win.create_text(ixmin-10, iymin, text = "%5.2f" % ymin, anchor = E)
   win.create_text(ixmin-10, iymax, text = "%5.2f" % ymax, anchor = E)
                                                         #  scaling coefficients
   ax = (ixmax - ixmin) / (xmax - xmin)                                 # x-axis
   bx = ixmin - ax * xmin
   ay = (iymax - iymin) / (ymax - ymin)                                 # y-axis
   by = iymin - ay * ymin
                                                                     # draw axes
   if (xmin * xmax < 0): win.create_line(Nint(bx), iymin, Nint(bx), iymax)   # y
   if (ymin * ymax < 0): win.create_line(ixmin, Nint(by), ixmax, Nint(by))   # x

   ix0 = Nint(ax * x[1] + bx); iy0 = Nint(ay * y[1] + by)            # 1st point
   for i in range(2, n+1):
      ix = Nint(ax * x[i] + bx); iy = Nint(ay * y[i] + by)           # new point
      win.create_line(ix0, iy0, ix, iy, fill = col)                  # draw line
      ix0 = ix; iy0 = iy                                            # save point
